/*
 * Copyright (c) 2003, Raymond Bosman
 * Copyright (c) 2003, Frederik Holljen
 * All Rights Reserved.
 *
 * See COPYING for licensing.
 */


header "post_include_hpp"
{
    #include "../antlr/CkASTNodeBase.hpp"
}

header "post_include_cpp"
{
}

// Global options
options
{
	mangleLiteralPrefix = "TK_"; // instead of TOKEN
	language="Cpp";
}

class CkTreeParser extends TreeParser;
options
{
	importVocab=Ck;
    ASTLabelType = "RefCkASTNodeBase";
}

{
}

program
   : #(PROGRAM (parseFunction)*)
   ;

parseFunction
	: #(FUNCTION primitive (IDENT (parameter)*  block | MAIN block))
	;

parameter
	: #(PARAM_DEF primitive IDENT)
	;


block
    : #(b:BLOCK (declaration | ex:expression | ret:returnExpr)*)
	;

declaration
	: #(DECLARATION (CONST)? prim:primitive varDeclaration (expression)?)
	;

varDeclaration
    : IDENT
    | #(ARRAY IDENT INT (INT)? )
    ;


returnExpr
	: #(RETURN expression)
	;

expression

	/* highest priority */
	: assign

	/* priority 6 */
    | #(orOp:OR lh:expression rh:expression) /* ** OR ** */
    
	/* priority 5 */
	| #(andOp:AND andlh:expression andrh:expression) /* ** AND ** */

   	/* priority 4 */
    | #(ltOp:LTHEN ltlh:expression ltrh:expression) /* ** LT ** */

	| #(leOp:LEQUAL lelh:expression lerh:expression) /* ** LEQUAL ** */

    | #(gtOp:GTHEN gtlh:expression gtrh:expression) /* ** GTHEN ** */

    | #(geOp:GEQUAL gelh:expression gerh:expression) /* ** GEQUAL ** */

    | #(eqOp:EQUAL eqlh:expression eqrh:expression) /* ** EQUAL ** */

    | #(neOp:NEQUAL nelh:expression nerh:expression) /* ** NEQUAL ** */

	/* priority 3 */
    | #(plusOp:PLUS pluslh:expression (plusrh:expression)?) /* * (UNARY) PLUS */

    | #(minusOp:MINUS minuslh:expression (minusrh:expression)?)

	/* priority 2 */
	| #(multOp:MULTIPLY multlh:expression multrh:expression)

	| #(divOp:DIVIDE divlh:expression divrh:expression)
        
	| #(modOp:MODULO modlh:expression modrh:expression)

	/* Priority 1 */
    | #(notOp:NOT notExpr:expression) /* ** NOT ** */

	// Todo: PLUS and MINUS are already defined priority 3.
	//| #(PLUS expression)

	/* Priority 0 */
	| operand
	| block
	| paren

	/* And expression functions */
	| getStatement
	| putStatement
	| ifStatement
	| whileStatement
	;

paren
	: #(PAREN expression)
	;

operand
	: variable
	| i:INT
	| c:CHAR
	| bt: TRUE
	| bf: FALSE
	;

variable
	: IDENT
    | #(ARRAY IDENT expression (expression)?)
    ;


assign
	:#(op:ASSIGN lh:expression rh:expression)
	;


programReturnType
	:	v:PRIM_VOID 
	|	b:PRIM_BOOLEAN 
	|	c:PRIM_CHAR 
	|	i:PRIM_INT 
	;


primitive
	:	"void"
	|	"bool"
	|	"char"
	|	"int"
	;



getStatement
	: #(getOp:GET (id:IDENT)+ )
	;

putStatement
    : #(putOp:PUT (expr:expression)+ )
	;


ifStatement
	: #(ifOp:IF expr:expression ifDo:block (elseDo:block)?)
	;

whileStatement
	: #(whileOp:WHILE expr:expression whileDo:block )
	;
